@@ -16,8 +16,9 @@
 
 //! please.rs a sudo-like clone that implements regex all over the place
 
-use pleaser::util::*;
+use pleaser::*;
 
+use std::convert::TryFrom;
 use std::fs::OpenOptions;
 use std::os::unix::fs::OpenOptionsExt;
 
@@ -31,10 +32,11 @@ use regex::Regex;
 
 use getopts::Options;
 
+use nix::sys::signal;
+use nix::sys::signal::Signal;
 use nix::sys::stat::fchmod;
 use nix::sys::wait::WaitStatus::Exited;
-use nix::unistd::{fchown, fork, gethostname, ForkResult};
-
+use nix::unistd::{fchown, fork, ForkResult};
 use users::*;
 
 /// return a path string to work on in /tmp
@@ -149,7 +151,7 @@ fn setup_temp_edit_file(
 fn build_exitcmd(entry: &EnvOptions, source_file: &str, edit_file: &str) -> Command {
     let cmd_re = Regex::new(r"\s+").unwrap();
 
-    let cmd_str = &(entry.exitcmd).as_ref().unwrap();
+    let cmd_str = &entry.exitcmd.as_ref().unwrap();
     let cmd_parts: Vec<&str> = cmd_re.split(&cmd_str).collect();
 
     if cmd_parts.is_empty() {
@@ -250,7 +252,7 @@ fn remove_tmp_edit(ro: &RunOptions, edit_file: &str) {
 fn rename_to_source(
     dir_parent_tmp: &str,
     source_file: &Path,
-    entry: &Result<EnvOptions, ()>,
+    entry: &EnvOptions,
     lookup_name: &User,
     dir_parent_tmp_file: &std::fs::File,
     target_uid: nix::unistd::Uid,
@@ -271,32 +273,42 @@ fn rename_to_source(
     )
     .unwrap();
 
-    fchmod(
-        dir_parent_tmp_file.as_raw_fd(),
-        if entry.as_ref().unwrap().edit_mode.is_some() {
-            nix::sys::stat::Mode::from_bits(entry.as_ref().unwrap().edit_mode.unwrap() as u32)
-                .unwrap()
-        } else {
-            nix::sys::stat::Mode::S_IRUSR | nix::sys::stat::Mode::S_IWUSR
-        },
-    )
-    .unwrap();
+    let edit_mode = if entry.edit_mode.is_some() {
+        match entry.edit_mode.as_ref().unwrap() {
+            EditMode::Mode(x) => nix::sys::stat::Mode::from_bits(*x as u32).unwrap(),
+            EditMode::Keep(_x) => match nix::sys::stat::stat(source_file) {
+                Ok(m) => nix::sys::stat::Mode::from_bits_truncate(m.st_mode),
+                _ => nix::sys::stat::Mode::from_bits(0o600).unwrap(),
+            },
+        }
+    } else {
+        nix::sys::stat::Mode::from_bits(0o600).unwrap()
+    };
 
-    if entry.as_ref().unwrap().exitcmd.is_some() {
-        let mut cmd = build_exitcmd(
-            &entry.clone().unwrap(),
-            &source_file.to_str().unwrap(),
-            &dir_parent_tmp,
-        );
-        let out = cmd.output().expect("could not execute");
-        io::stdout().write_all(&out.clone().stdout).unwrap();
-        io::stderr().write_all(&out.clone().stderr).unwrap();
-        if !out.status.success() {
-            println!("Aborting as exitcmd was non-zero, removing tmp file");
-            if nix::unistd::unlink(dir_parent_tmp).is_err() {
-                println!("Could not remove tmp file either, giving up");
+    fchmod(dir_parent_tmp_file.as_raw_fd(), edit_mode).unwrap();
+
+    if entry.exitcmd.is_some() {
+        let mut cmd = build_exitcmd(&entry, &source_file.to_str().unwrap(), &dir_parent_tmp);
+        match cmd.output() {
+            Err(x) => {
+                println!("Aborting as exitcmd was non-zero when executing, removing tmp file:");
+                println!("{}", x);
+                if nix::unistd::unlink(dir_parent_tmp).is_err() {
+                    println!("Could not remove tmp file either, giving up");
+                }
+                std::process::exit(1);
+            }
+            Ok(out) => {
+                io::stdout().write_all(&out.stdout).unwrap();
+                io::stderr().write_all(&out.stderr).unwrap();
+                if !out.status.success() {
+                    println!("Aborting as exitcmd was non-zero, removing tmp file");
+                    if nix::unistd::unlink(dir_parent_tmp).is_err() {
+                        println!("Could not remove tmp file either, giving up");
+                    }
+                    std::process::exit(1);
+                }
             }
-            std::process::exit(1);
         }
     }
 
@@ -325,6 +337,24 @@ fn edit_file_to_memory(source_file: &Path, edit_file: &str) -> Result<String, st
     file_data
 }
 
+extern "C" fn handle_sigtstp(
+    child: libc::c_int,
+    info: *mut libc::siginfo_t,
+    _th: *mut libc::c_void,
+) {
+    let signal = Signal::try_from(child).unwrap();
+    unsafe {
+        // don't have a handy definition for "5" SI_MESGQ
+        if signal == Signal::SIGCHLD && (*info).si_code == 5 {
+            signal::kill(
+                nix::unistd::Pid::from_raw(std::process::id() as i32),
+                Signal::SIGTSTP,
+            )
+            .unwrap();
+        }
+    }
+}
+
 /// entry point
 fn main() {
     let args: Vec<String> = std::env::args().collect();
@@ -333,9 +363,8 @@ fn main() {
     let mut ro = RunOptions::new();
     let original_uid = get_current_uid();
     let original_user = get_user_by_uid(original_uid).unwrap();
-    let original_gid = original_user.primary_group_id();
     ro.name = original_user.name().to_string_lossy().to_string();
-    ro.acl_type = ACLTYPE::EDIT;
+    ro.acl_type = Acltype::Edit;
     ro.syslog = true;
     let mut vec_eo: Vec<EnvOptions> = vec![];
 
@@ -353,7 +382,7 @@ fn main() {
     }
 
     general_options(&mut ro, args, &service);
-    if ro.target == "" {
+    if ro.target.is_empty() {
         ro.target = "root".to_string();
     }
     ro.command = ro.new_args.join(" ");
@@ -372,46 +401,28 @@ fn main() {
         std::process::exit(1);
     }
 
-    let mut buf = [0u8; 64];
-    ro.hostname = gethostname(&mut buf)
-        .expect("Failed getting hostname")
-        .to_str()
-        .expect("Hostname wasn't valid UTF-8")
-        .to_string();
     let entry = can(&vec_eo, &ro);
 
-    match &entry {
-        Err(_) => {
-            log_action(&service, "deny", &ro, &original_command.join(" "));
-            println!(
-                "You may not edit \"{}\" on {} as {}",
-                &ro.command, &ro.hostname, &ro.target
-            );
-            std::process::exit(1);
-        }
-        Ok(x) => {
-            ro.syslog = x.syslog;
-            if !x.permit {
-                log_action(&service, "deny", &ro, &original_command.join(" "));
-                println!(
-                    "You may not edit \"{}\" on {} as {}",
-                    &ro.command, &ro.hostname, &ro.target
-                );
-                std::process::exit(1);
-            }
-            // check if a reason was given
-            if x.permit && x.reason && ro.reason.is_none() {
-                log_action(&service, "no_reason", &ro, &original_command.join(" "));
-                println!(
-                    "Sorry but no reason was given to edit \"{}\" on {} as {}",
-                    &ro.command, &ro.hostname, &ro.target
-                );
-                std::process::exit(1);
-            }
-        }
+    ro.syslog = entry.syslog;
+    if !entry.permit {
+        log_action(&service, "deny", &ro, &original_command.join(" "));
+        println!(
+            "You may not edit \"{}\" on {} as {}",
+            &ro.command, &ro.hostname, &ro.target
+        );
+        std::process::exit(1);
+    }
+    // check if a reason was given
+    if entry.permit && entry.reason && ro.reason.is_none() {
+        log_action(&service, "no_reason", &ro, &original_command.join(" "));
+        println!(
+            "Sorry but no reason was given to edit \"{}\" on {} as {}",
+            &ro.command, &ro.hostname, &ro.target
+        );
+        std::process::exit(1);
     }
 
-    if !challenge_password(&ro, entry.clone().unwrap(), &service) {
+    if !challenge_password(&ro, &entry, &service) {
         log_action(&service, "deny", &ro, &original_command.join(" "));
         std::process::exit(1);
     }
@@ -428,45 +439,62 @@ fn main() {
 
     let source_file = Path::new(&ro.new_args[0]);
 
-    if !esc_privs() {
+    if !drop_privs(&ro) {
         std::process::exit(1);
     }
-    let edit_file = &setup_temp_edit_file(&service, source_file, &ro, target_uid, target_gid);
 
-    set_environment(&ro, &original_user, original_uid, &lookup_name);
+    set_environment(&ro, &entry, &original_user, original_uid, &lookup_name);
+    let edit_file = &setup_temp_edit_file(&service, source_file, &ro, target_uid, target_gid);
 
     std::env::set_var("PLEASE_EDIT_FILE", edit_file.to_string());
     std::env::set_var("PLEASE_SOURCE_FILE", source_file.to_str().unwrap());
 
-    if !esc_privs() {
-        std::process::exit(1);
-    }
-
     let mut good_edit = false;
 
-    match unsafe { fork() } {
-        Ok(ForkResult::Parent { .. }) => match nix::sys::wait::wait() {
-            Ok(Exited(_pid, ret)) if ret == 0 => {
-                good_edit = true;
-            }
-            Ok(_) => {}
-            Err(_x) => {}
-        },
-        Ok(ForkResult::Child) => {
-            // drop privileges and execute editor
-            let editor = get_editor();
+    let sig_action = signal::SigAction::new(
+        signal::SigHandler::SigAction(handle_sigtstp),
+        signal::SaFlags::SA_RESTART,
+        signal::SigSet::all(),
+    );
 
-            if !set_privs(
-                &ro.name,
-                nix::unistd::Uid::from_raw(original_uid),
-                nix::unistd::Gid::from_raw(original_gid),
-            ) {
-                println!("I cannot set privs. Exiting as not installed correctly.");
+    match unsafe { fork() } {
+        Ok(ForkResult::Parent { .. }) => {
+            unsafe {
+                signal::sigaction(signal::SIGCHLD, &sig_action).unwrap();
+            };
+
+            match nix::sys::wait::wait() {
+                Ok(Exited(_pid, ret)) if ret == 0 => {
+                    good_edit = true;
+                }
+                Ok(_) => {}
+                Err(_x) => {}
+            }
+            unsafe {
+                signal::signal(signal::SIGCHLD, signal::SigHandler::SigDfl).unwrap();
+            };
+        }
+        Ok(ForkResult::Child) => {
+            if !esc_privs() {
+                std::process::exit(1);
+            }
+            if !set_privs(&ro.name, ro.original_uid, ro.original_gid) {
                 std::process::exit(1);
             }
 
+            let editor = get_editor();
+
             nix::sys::stat::umask(ro.old_umask.unwrap());
 
+            if ro.old_envs.is_some() {
+                for (key, _) in std::env::vars() {
+                    std::env::remove_var(key);
+                }
+                for (key, val) in ro.old_envs.as_ref().unwrap().iter() {
+                    std::env::set_var(key, val);
+                }
+            }
+
             let args: Vec<&str> = editor.as_str().split(' ').collect();
             if args.len() == 1 {
                 Command::new(editor.as_str()).arg(&edit_file).exec();
@@ -482,10 +510,6 @@ fn main() {
         Err(_) => println!("Fork failed"),
     }
 
-    if !drop_privs(&ro) {
-        std::process::exit(1);
-    }
-
     if !good_edit {
         println!("Exiting as editor or child did not close cleanly.");
         std::process::exit(1);
@@ -495,6 +519,7 @@ fn main() {
     log_action(&service, "permit", &ro, &original_command.join(" "));
     let dir_parent_tmp =
         source_tmp_file_name(&source_file, format!("{}.copy", service).as_str(), &ro.name);
+
     let file_data = edit_file_to_memory(&source_file, &edit_file);
 
     // become the target user and create file
@@ -515,6 +540,3 @@ fn main() {
         target_gid,
     );
 }
-
-#[cfg(test)]
-mod test {}
